from typing import Dict, Union

from okx_quant.utils.request_utils import AccountStatus, RequestMethod, RequestUtils
from okx_quant.utils.config_utils import ConfigUtils, APIConfig
from okx_quant.future.future import OrderType, TradeSide, MarginType

class Spot:
    def __init__(self, api_config: APIConfig, config: dict):
        self.config: dict = config
        self.api_config: APIConfig = api_config
        self.request_utils: RequestUtils = RequestUtils(api_config, config)
        self.account_status: AccountStatus = AccountStatus.Demo if config[
            'mode']['demo'] else AccountStatus.Trade

    def trade(self, spot_id: str, side: TradeSide, order_vol: float,
                      order_type: OrderType = OrderType.MARKET,
                      order_price: float = 0.0,
                      order_id: str = "") -> Union[Dict, None]:
        params = {
            "instId": spot_id,
            "tdMode": 'cash',
            "side": side.value,
            "ordType": order_type.value,
            "sz": f"{order_vol:.10f}"
        }

        if order_id != "":
            params["clOrdId"] = order_id

        if order_type != OrderType.MARKET:
            params["px"] = f"{order_price:.10f}"

        resp = self.request_utils.request(RequestMethod.POST,
                                          "/api/v5/trade/order", body=params, auth=True,
                                          account_status=self.account_status)
        return resp







from dataclasses import dataclass

import toml


@dataclass
class APIConfig:
    api_key: str
    secret_key: str
    passphrase: str
    base_url: str = 'https://www.dojqutmbiuuo.com'


class ConfigUtils:
    def __init__(self, config_path: str):
        self.config: dict = toml.load(config_path)
        self.api_config: APIConfig = self.read_config()

    def read_config(self) -> APIConfig:
        api_key: str = self.config['api']['api_key']
        secret_key: str = self.config['api']['secret_key']
        passphrase: str = self.config['api']['passphrase']
        base_url: str = 'https://www.dojqutmbiuuo.com'
        if 'base_url' in self.config:
            base_url = self.config['base_url']
        return APIConfig(api_key, secret_key, passphrase, base_url)



import enum
from typing import Dict, List, Union
from dataclasses import dataclass

from loguru import logger

from okx_quant.utils.config_utils import APIConfig
from okx_quant.utils.request_utils import RequestUtils, RequestMethod


class GridType(enum.Enum):
    SPOT = 'grid'
    FUTURE = 'contract_grid'


class OrderStatus(enum.Enum):
    LIVE = 'live'
    FILLED = 'filled'


@dataclass
class GridBotData:
    instance_name: str
    grid_type: GridType
    investment: float
    grid_num: int
    arbitrage_num: int
    grid_profit: float
    float_profit: float
    max_price: float
    min_price: float
    total_pnl: float
    # for future grid bots
    lever: int
    liq_price: float
    # order lists
    buy_lists: List[float]
    sell_lists: List[float]


class GridBot:
    def __init__(self, api_config: APIConfig, config: dict):
        self.api_config: APIConfig = api_config
        self.config: dict = config
        self.request_utils: RequestUtils = RequestUtils(api_config, config)
        self.grid_bots: Dict[str, GridBotData] = {}
        self.sync_once()

    def get_grid_order_list(self, grid_type: GridType) -> dict:
        params = {
            'algoOrdType': grid_type.value,
        }
        uri = "/api/v5/tradingBot/grid/orders-algo-pending"
        result = self.request_utils.request(
            RequestMethod.GET, uri, params=params, auth=True
        )
        return result

    def get_grid_order_details(self, grid_type: GridType, algo_id: Union[str, int]) -> dict:
        params = {
            'algoOrdType': grid_type.value,
            'algoId': algo_id,
        }
        uri = "/api/v5/tradingBot/grid/orders-algo-details"
        result = self.request_utils.request(
            RequestMethod.GET, uri, params=params, auth=True
        )
        return result

    def get_grid_sub_orders(self, grid_type: GridType, algo_id: Union[str, int],
                            order_status: OrderStatus) -> dict:
        params = {
            'algoOrdType': grid_type.value,
            'algoId': algo_id,
            'type': order_status.value,
        }
        uri = "/api/v5/tradingBot/grid/sub-orders"
        result = self.request_utils.request(
            RequestMethod.GET, uri, params=params, auth=True
        )
        return result

    def get_grid_positions(self, grid_type: GridType, algo_id: Union[str, int]) -> dict:
        params = {
            'algoOrdType': grid_type.value,
            'algoId': algo_id,
        }
        uri = "/api/v5/tradingBot/grid/positions"
        result = self.request_utils.request(
            RequestMethod.GET, uri, params=params, auth=True
        )
        return result

    def sync_once(self):
        grid_types = [GridType.SPOT, GridType.FUTURE]
        for grid_type in grid_types:
            # get order details
            grid_order_list = self.get_grid_order_list(grid_type)
            grid_order_list = grid_order_list['data']
            for grid_order in grid_order_list:
                bot_id = grid_order['algoId']
                self.grid_bots[bot_id] = GridBotData(
                    grid_order['instId'],
                    grid_type,
                    grid_order['investment'],
                    grid_order['gridNum'],
                    grid_order['arbitrageNum'],
                    grid_order['gridProfit'],
                    grid_order['floatProfit'],
                    grid_order['maxPx'],
                    grid_order['minPx'],
                    grid_order['totalPnl'],
                    grid_order['lever'],
                    grid_order['liqPx'],
                    [], []
                )

            # get order sub orders
            for bot_id in self.grid_bots.keys():
                sub_orders = self.get_grid_sub_orders(self.grid_bots[bot_id].grid_type, bot_id, OrderStatus.LIVE)
                sub_orders = sub_orders['data']
                for sub_order in sub_orders:
                    if sub_order['side'] == 'buy':
                        self.grid_bots[bot_id].buy_lists.append(sub_order['px'])
                    elif sub_order['side'] == 'sell':
                        self.grid_bots[bot_id].sell_lists.append(sub_order['px'])



from typing import Union, Dict
from enum import Enum

from okx_quant.utils.config_utils import APIConfig, ConfigUtils
from okx_quant.utils.request_utils import RequestUtils, RequestMethod, AccountStatus

class TransferType(Enum):
    NORMAL = "0"
    MAIN_TO_SUB = "1"
    SUB_TO_MAIN = "2"     # for main api
    SUB_TO_MAIN_2 = "3"   # for sub api
    SUB_TO_SUB = "4"      # for sub api

class AccountType(Enum):
    FUNDING_ACC = "6"
    TRADING_ACC = "18"


class Account:
    def __init__(self, api_config: APIConfig, config: dict):
        self.api_config: APIConfig = api_config
        self.config: dict = config
        self.request_utils: RequestUtils = RequestUtils(api_config, config)
        self.account_status: AccountStatus = AccountStatus.Demo if config[
            'mode']['demo'] else AccountStatus.Trade

    def get_balance(self, currency: str) -> Union[Dict, None]:
        params = {"ccy": currency}
        uri = "/api/v5/account/balance"
        result = self.request_utils.request(
            RequestMethod.GET, uri=uri, params=params, auth=True,
            account_status=self.account_status
        )
        return result
    
    def get_assets(self, currency: str) -> Union[Dict, None]:
        params = {
            "ccy": currency,
        }
        uri = "/api/v5/asset/asset-valuation"
        result = self.request_utils.request(
            RequestMethod.GET, uri=uri, params=params, auth=True,
            account_status=self.account_status
        )
        return result
    
    def transfer(self, amount: float,
                from_account: AccountType, to_account: AccountType, 
                currency: str = 'USDT',
                transfer_type: TransferType = TransferType.NORMAL, 
                sub_account: str = '') -> Union[Dict, None]:
        params = {
            "type": transfer_type.value,
            "ccy": currency,
            "amt": f"{amount:.10f}",
            "from": from_account.value,
            "to": to_account.value,
        }

        if type in [TransferType.MAIN_TO_SUB, TransferType.SUB_TO_MAIN, TransferType.SUB_TO_SUB]:
            params["subAcct"] = sub_account
        
        uri = "/api/v5/asset/transfer"
        result = self.request_utils.request(
            RequestMethod.POST, uri=uri, body=params, auth=True,
            account_status=self.account_status
        )
        return result



import requests
import base64
import json
import time
import hmac

from urllib.parse import urljoin
from enum import Enum
from typing import Union

from okx_quant.utils.config_utils import ConfigUtils, APIConfig


class RequestMethod(Enum):
    GET: str = 'GET'
    POST: str = 'POST'

class AccountStatus(Enum):
    Trade: str = 'TRADE'
    Demo: str = 'DEMO'

class RequestUtils:
    def __init__(self, api_config: APIConfig, config: dict):
        self.api_config: APIConfig = api_config
        self.config: dict = config

    def request(self, method: RequestMethod, uri: str, params: dict = None,
                body: dict = None, headers: dict = None, auth: bool = False, 
                account_status: AccountStatus = AccountStatus.Trade) -> Union[dict, None]:
        if params:
            query: str = "&".join(
                ["{}={}".format(k, params[k]) for k in sorted(params.keys())]
            )
            uri += "?" + query
        url: str = urljoin(self.api_config.base_url, uri)

        if auth:
            timestamp: str = (
                    str(time.time()).split(".")[0]
                    + "."
                    + str(time.time()).split(".")[1][:3]
            )
            if body:
                body: str = json.dumps(body)
            else:
                body: str = ""
            message: str = str(timestamp) + str.upper(method.value) + uri + str(body)
            mac: hmac.HMAC = hmac.new(
                bytes(self.api_config.secret_key, encoding="utf-8"),
                bytes(message, encoding="utf-8"),
                digestmod="sha256",
            )
            digest: bytes = mac.digest()
            sign: bytes = base64.b64encode(digest)

            if not headers:
                headers = {}
            headers["Content-Type"] = "application/json"
            headers["OK-ACCESS-KEY"] = self.api_config.api_key
            headers["OK-ACCESS-SIGN"] = sign
            headers["OK-ACCESS-TIMESTAMP"] = str(timestamp)
            headers["OK-ACCESS-PASSPHRASE"] = self.api_config.passphrase
            # add special header for demo mode
            if account_status == AccountStatus.Demo:
                headers["x-simulated-trading"] = '1'

        result: dict = requests.request(method.value, url, data=body, headers=headers, timeout=10).json()
        if result.get('code') and result.get('code') != '0':
            print(f"Err occurs while requesting {uri}: ")
            print(result)
            return None
        return result



from datetime import datetime

def str_to_timestamp_ms(date_str, format_str="%Y-%m-%d"):
    dt = datetime.strptime(date_str, format_str)
    timestamp_ms = int(dt.timestamp() * 1000)
    return timestamp_ms

def timestamp_ms_to_str(timestamp_ms, format_str="%Y-%m-%d"):
    timestamp_s = timestamp_ms / 1000
    dt = datetime.fromtimestamp(timestamp_s)
    return dt.strftime(format_str)


import enum
from typing import Dict, Union

from okx_quant.utils.request_utils import AccountStatus, RequestMethod, RequestUtils
from okx_quant.utils.config_utils import ConfigUtils, APIConfig


class MarginType(enum.Enum):
    CROSS = "cross"
    ISOLATED = "isolated"


class OrderType(enum.Enum):
    LIMIT = "limit"
    MARKET = "market"
    POST_ONLY = "post_only"  # only for maker order
    # advanced type, see official doc
    FOK = "fok"
    IOC = "ioc"
    OPTIMAL_LIMIT_IOC = "optimal_limit_ioc"
    MMP = "mmp"
    MMP_AND_POST_ONLY = "mmp_and_post_only"


class TradeSide(enum.Enum):
    # buy & sell in long_short_mode
    BUY = "buy"
    SELL = "sell"
    # long & short in net_mode
    LONG = "long"
    SHORT = "short"


class MarginAdjustType(enum.Enum):
    ADD = "add"
    REDUCE = "reduce"


class Future:
    def __init__(self, api_config: APIConfig, config: dict):
        self.config: dict = config
        self.api_config: APIConfig = api_config
        self.request_utils: RequestUtils = RequestUtils(api_config, config)
        self.account_status: AccountStatus = AccountStatus.Demo if config[
            'mode']['demo'] else AccountStatus.Trade

    def open_position(self, future_id: str, side: TradeSide, order_vol: float,
                      order_type: OrderType = OrderType.MARKET,
                      margin_type: MarginType = MarginType.ISOLATED,
                      order_price: float = 0.0,
                      order_id: str = "", lever: float = -1) -> Union[Dict, None]:
        params = {
            "instId": future_id,
            "tdMode": margin_type.value,
            "side": side.value,
            "ordType": order_type.value,
            "sz": f"{order_vol:.10f}"
        }

        if order_id != "":
            params["clOrdId"] = order_id

        if order_type != OrderType.MARKET:
            params["px"] = f"{order_price:.10f}"

        if lever != -1 and lever >= 0:
            self.set_future_lever(future_id, lever, margin_type)

        resp = self.request_utils.request(RequestMethod.POST,
                                          "/api/v5/trade/order", body=params, auth=True,
                                          account_status=self.account_status)
        return resp

    def query_position(self, future_id: str) -> Union[Dict, None]:
        params = {
            "instId": future_id
        }

        resp = self.request_utils.request(
            RequestMethod.GET, "/api/v5/account/positions", params=params, auth=True, account_status=self.account_status)
        return resp

    def has_position(self, future_id: str) -> bool:
        resp = self.query_position(future_id)
        px_str = resp['data'][0]['avgPx']
        px = float(px_str) if px_str != "" else 0.0
        return px > 0.0

    def close_position(self, future_id: str,
                       margin_type: MarginType = MarginType.ISOLATED,
                       order_id: str = "") -> Union[Dict, None]:
        params = {
            "instId": future_id,
            "mgnMode": margin_type.value,
        }

        if order_id != "":
            params["clOrdId"] = order_id

        resp = self.request_utils.request(RequestMethod.POST, "/api/v5/trade/close-position",
                                          body=params, auth=True, account_status=self.account_status)
        return resp

    def set_future_lever(self, future_id: str, lever: float,
                         margin_type: MarginType = MarginType.ISOLATED) -> Union[Dict, None]:
        params = {
            "instId": future_id,
            "lever": f"{lever:.2f}",
            "mgnMode": margin_type.value,
        }

        resp = self.request_utils.request(
            RequestMethod.POST, "/api/v5/account/set-leverage", body=params, auth=True, account_status=self.account_status)
        return resp

    def adjust_margin(self, future_id: str, adjust_type: MarginAdjustType,
                      amount: float, ccy: str = 'USDT') -> Union[Dict, None]:
        params = {
            "instId": future_id,
            "posSide": "net",  # pos side should always be net under default mode
            "type": adjust_type.value,
            "amt": f"{amount:.10f}",
        }

        if ccy != 'USDT':
            params['ccy'] = ccy

        resp = self.request_utils.request(
            RequestMethod.POST, "/api/v5/account/position/margin-balance", body=params, auth=True, account_status=self.account_status)
        return resp

    def get_max_trade_size(self, future_id: str, lever: float = -1, order_price: float = -1,
                        margin_type: MarginType = MarginType.ISOLATED) -> Union[Dict, None]:
        params = {
            "instId": future_id,
            "tdMode": margin_type.value,
        }

        if order_price != -1:
            params["px"] = f"{order_price: 10f}"

        if lever != -1:
            self.set_future_lever(future_id, lever)

        resp = self.request_utils.request(
            RequestMethod.GET, "/api/v5/account/max-size",
            params=params,
            auth=True,
            account_status=self.account_status
        )
        return resp

    def get_max_margin(self, future_id: str,
                order_price: float = -1,
                margin_type: MarginType = MarginType.ISOLATED) -> Union[Dict, None]:
        params = {
            "instId": future_id,
            "tdMode": margin_type.value,
        }

        if order_price != -1:
            params["px"] = f"{order_price: 10f}"
        
        resp = self.request_utils.request(
            RequestMethod.GET, "/api/v5/account/max-avail-size",
            params=params,
            auth=True,
            account_status=self.account_status
        )
        return resp


from datetime import datetime, timedelta
import time
import pandas as pd

from okx_quant.utils.time_utils import str_to_timestamp_ms, timestamp_ms_to_str
from okx_quant.utils.request_utils import RequestUtils, RequestMethod
from okx_quant.utils.config_utils import APIConfig



class Kline:
    def __init__(self, api_config: APIConfig, config: dict):
        self.config: dict = config
        self.api_config: APIConfig = api_config
        self.request_utils: RequestUtils = RequestUtils(api_config, config)

    def get_history_kline(self, symbol: str, begin: str, end: str, period: str='1D',
                        limit: int=100, time_seg: float = 0.1):
        all_klines = []
        format_str = "%Y-%m-%d" if period == '1D' else "%Y-%m-%d %H:%M:%S"
        begin_dt = datetime.strptime(begin, format_str)
        end_dt = datetime.strptime(end, format_str)
        
        while end_dt >= begin_dt:
            diff = abs((end_dt - begin_dt).days)
            if period == '1D':
                params = {
                    "instId": symbol,
                    "before": str_to_timestamp_ms(begin_dt.strftime(format_str), format_str),
                    "after": str_to_timestamp_ms((begin_dt + timedelta(days=min(diff, limit))).strftime(format_str), format_str),
                    "bar": period,
                    "limit": limit,
                }
            elif period == '1H':
                params = {
                    "instId": symbol,
                    "before": str_to_timestamp_ms(begin_dt.strftime(format_str), format_str),
                    "after": str_to_timestamp_ms((begin_dt + timedelta(hours=min(diff * 24, limit))).strftime(format_str), format_str),
                    "bar": period,
                    "limit": limit,
                }
            
            try:
                result = self.request_utils.request(RequestMethod.GET, "/api/v5/market/history-candles", params=params, auth=True)
                klines = result['data']
                all_klines.extend(klines)
                
                if period == '1D':
                    begin_dt += timedelta(days=limit-1)
                elif period == '1H':
                    begin_dt += timedelta(hours=limit-1)
                time.sleep(time_seg)
                
            except Exception as e:
                print(f"Error fetching data: {e}")
                break
        
        df = None
        try:
            if all_klines:
                cols = ['timestamp', 'open', 'high', 'low', 'close', 'vol', 'volCcy', 'volCcyQuote', 'confirm']
                kline_dict = {col: [] for col in cols}
                
                for kline in all_klines:
                    kline[0] = timestamp_ms_to_str(int(kline[0]), "%Y-%m-%d %H:%M:%S" if period == '1H' else "%Y-%m-%d")
                    for idx, col in enumerate(cols):
                        kline_dict[col].append(kline[idx] if col == 'timestamp' else float(kline[idx]))
                
                df = pd.DataFrame(kline_dict)
                df = df.sort_values('timestamp').drop_duplicates(subset=['timestamp']).reset_index(drop=True)
                
        except Exception as e:
            print(f"Error processing data: {e}")
        
        return df



from setuptools import setup, find_packages

setup(
    name="okx_quant",
    version="0.0.14",
    packages=find_packages(),
    author="openhe",
    author_email="hezhewen2004@gmail.com",
    description="okx quant",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url='https://github.com/openhe-hub/okx-quant',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



from typing import Union, Dict
from enum import Enum

from okx_quant.utils.request_utils import RequestMethod, RequestUtils
from okx_quant.utils.config_utils import APIConfig, ConfigUtils


class EarnOperateType(Enum):
    PURCHASE = "purchase"
    REDEMPT = "redempt"


class Earn():
    def __init__(self, api_config: APIConfig, config: dict):
        self.config: dict = config
        self.api_config: APIConfig = api_config
        self.request_utils: RequestUtils = RequestUtils(api_config, config)
    

    def get_balance(self, currency: str = 'USDT') -> Union[Dict, None]:
        params = {
            "ccy": currency,
        }

        uri = "/api/v5/finance/savings/balance"
        resp = self.request_utils.request(
            method=RequestMethod.GET,
            uri=uri, 
            params=params,
            auth=True
        )
        return resp

    def trade(self, amount: float, side: EarnOperateType,
            currency: str= 'USDT', rate: float = 0.01) -> Union[Dict, None]:
        params = {
            "ccy": currency,
            "amt": f"{amount:.10f}",
            "side": side.value,
            "rate": f"{rate:.10f}",
        }

        uri = "/api/v5/finance/savings/purchase-redempt"
        resp = self.request_utils.request(
            method=RequestMethod.POST,
            uri=uri, 
            body=params,
            auth=True
        )
        return resp
        